/*
 * Project 64 - A Nintendo 64 emulator.
 *
 * (c) Copyright 2001 zilmar (zilmar@emulation64.com) and 
 * Jabo (jabo@emulation64.com).
 *
 * pj64 homepage: www.pj64.net
 *
 * Permission to use, copy, modify and distribute Project64 in both binary and
 * source form, for non-commercial purposes, is hereby granted without fee,
 * providing that this license information and copyright notice appear with
 * all copies and any derived work.
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event shall the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Project64 is freeware for PERSONAL USE only. Commercial users should
 * seek permission of the copyright holders first. Commercial use includes
 * charging money for Project64 or software derived from Project64.
 *
 * The copyright holders request that bug fixes and improvements to the code
 * should be forwarded to them so if they want them.
 *
 */
#include <windows.h>
#include <stdio.h>
#include "main.h"
#include "CPU.h"
#include "plugin.h"

char RspDLL[100], GfxDLL[100], AudioDLL[100],ControllerDLL[100], * PluginNames[MaxDlls];
HINSTANCE hAudioDll, hControllerDll, hGfxDll, hRspDll;
DWORD PluginCount, RspTaskValue, AudioIntrReg;
WORD RSPVersion,ContVersion;
HANDLE hAudioThread = NULL;
//GFXDEBUG_INFO GFXDebug;
//RSPDEBUG_INFO RspDebug;
CONTROL Controllers[4];
extern DWORD RSPSECTIONS;

BOOL PluginsChanged ( void );
BOOL ValidPluginVersion ( PLUGIN_INFO * PluginInfo );

BOOL LoadAudioDll(char * AudioDll) {
	PLUGIN_INFO PluginInfo;

	AiGetDllInfo(&PluginInfo);
	if (!ValidPluginVersion(&PluginInfo) || PluginInfo.MemoryBswaped == FALSE) { return FALSE; }

	return TRUE;
}

BOOL LoadRSPDll(char * RspDll) {
	PLUGIN_INFO PluginInfo;
	GetDllInfo=RspGetDllInfo;
	if (GetDllInfo == NULL) { return FALSE; }
	
	GetDllInfo(&PluginInfo);
	if (!ValidPluginVersion(&PluginInfo) || PluginInfo.MemoryBswaped == FALSE) { return FALSE; }
	RSPVersion = PluginInfo.Version;
	if (RSPVersion == 1) { RSPVersion = 0x0100; }

	DoRspCycles = RspDoRspCycles;
	if (DoRspCycles == NULL) { return FALSE; }
	InitiateRSP_1_0 = NULL;
	InitiateRSP_1_1 = NULL;
	if (RSPVersion == 0x100) {
		DisplayError("Bad RSP lib!");
	}
	if (RSPVersion == 0x101) {
		InitiateRSP_1_1 = RspInitiateRSP;
		if (InitiateRSP_1_1 == NULL) { return FALSE; }
	}
	RSPRomClosed = RspRomClosed;
	if (RSPRomClosed == NULL) { return FALSE; }
	RSPCloseDLL = RspCloseDLL;
	if (RSPCloseDLL == NULL) { return FALSE; }
	RSPDllConfig = RspDllConfig;
	return TRUE;
}

void UnLoadRSPDll(void) {
	GetDllInfo=NULL;
	RSPVersion=NULL;
	DoRspCycles=NULL;
	RSPRomClosed=NULL;
	RSPCloseDLL=NULL;
	RSPDllConfig=NULL;
}


void SetupPlugins (void) {
	//HMENU hMenu = GetMenu(hMainWindow);

	ShutdownPlugins();
	//GetCurrentDlls();

	//EnableMenuItem(hMenu,ID_FILE_OPEN_ROM,MFS_GRAYED|MF_BYCOMMAND);		
	//EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_GFX,MFS_GRAYED|MF_BYCOMMAND);		
	//EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_RSP,MFS_GRAYED|MF_BYCOMMAND);		
	//EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_CONTROL,MFS_GRAYED|MF_BYCOMMAND);		

	if (!LoadAudioDll(AudioDLL) ) {
		//AiCloseDLL       = NULL;
		//AiDacrateChanged = NULL;
		//AiLenChanged     = NULL;
		//AiReadLength     = NULL;
		//AiUpdate         = NULL;
		//InitiateAudio    = NULL;
		//ProcessAList     = NULL;
		//AiRomClosed      = NULL;
		DisplayError("Could Not load Audio plugin ...");
	} else {
		AUDIO_INFO AudioInfo;
				
		AudioInfo.hwnd = NULL; //hWnd;
		AudioInfo.hinst = hInst;
		AudioInfo.MemoryBswaped = TRUE;
		AudioInfo.HEADER = RomHeader;
		AudioInfo.RDRAM = N64MEM;
		AudioInfo.DMEM = DMEM;
		AudioInfo.IMEM = IMEM;
		AudioInfo.MI__INTR_REG = &AudioIntrReg;	
		AudioInfo.AI__DRAM_ADDR_REG = &AI_DRAM_ADDR_REG;	
		AudioInfo.AI__LEN_REG = &AI_LEN_REG;	
		AudioInfo.AI__CONTROL_REG = &AI_CONTROL_REG;	
		AudioInfo.AI__STATUS_REG = &AI_STATUS_REG;	
		AudioInfo.AI__DACRATE_REG = &AI_DACRATE_REG;	
		AudioInfo.AI__BITRATE_REG = &AI_BITRATE_REG;	
		AudioInfo.CheckInterrupts = AiCheckInterrupts;
		AudioInfo.mod = &mod;
		if (!InitiateAudio(AudioInfo)) {
			//AiCloseDLL       = NULL;
			//AiDacrateChanged = NULL;
			//AiLenChanged     = NULL;
			//AiReadLength     = NULL;
			//AiUpdate         = NULL;
			//InitiateAudio    = NULL;
			//ProcessAList     = NULL;
			//AiRomClosed      = NULL;
			DisplayError("Failed to Initilize Audio");
		}
		//if (AiUpdate()) { 
		//	DWORD ThreadID;
		//	hAudioThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)AudioThread, (LPVOID)NULL,0, &ThreadID);			
		//}
	}

	if (!LoadRSPDll(RspDLL)) { 
		DisplayError("Could Not load RSP plugin ...");
		return;
	} else { 
		RSP_INFO_1_0 RspInfo10;
		RSP_INFO_1_1 RspInfo11;

		RspInfo10.CheckInterrupts = CheckInterrupts;
		RspInfo11.CheckInterrupts = CheckInterrupts;
		RspInfo10.ProcessDlist = NULL; //ProcessDList;
		RspInfo11.ProcessDlist = NULL; //ProcessDList;
		RspInfo10.ProcessAlist = ProcessAList;
		RspInfo11.ProcessAlist = ProcessAList;
		RspInfo10.ProcessRdpList = NULL; //ProcessRDPList;
		RspInfo11.ProcessRdpList = NULL; //ProcessRDPList;
		RspInfo11.ShowCFB = NULL; //ShowCFB;

		RspInfo10.hInst = hInst;
		RspInfo11.hInst = hInst;
		RspInfo10.RDRAM = N64MEM;
		RspInfo11.RDRAM = N64MEM;
		RspInfo10.DMEM = DMEM;
		RspInfo11.DMEM = DMEM;
		RspInfo10.IMEM = IMEM;
		RspInfo11.IMEM = IMEM;
		RspInfo10.MemoryBswaped = FALSE;
		RspInfo11.MemoryBswaped = FALSE;

		RspInfo10.MI__INTR_REG = &MI_INTR_REG;
		RspInfo11.MI__INTR_REG = &MI_INTR_REG;
			
		RspInfo10.SP__MEM_ADDR_REG = &SP_MEM_ADDR_REG;
		RspInfo11.SP__MEM_ADDR_REG = &SP_MEM_ADDR_REG;
		RspInfo10.SP__DRAM_ADDR_REG = &SP_DRAM_ADDR_REG;
		RspInfo11.SP__DRAM_ADDR_REG = &SP_DRAM_ADDR_REG;
		RspInfo10.SP__RD_LEN_REG = &SP_RD_LEN_REG;
		RspInfo11.SP__RD_LEN_REG = &SP_RD_LEN_REG;
		RspInfo10.SP__WR_LEN_REG = &SP_WR_LEN_REG;
		RspInfo11.SP__WR_LEN_REG = &SP_WR_LEN_REG;
		RspInfo10.SP__STATUS_REG = &SP_STATUS_REG;
		RspInfo11.SP__STATUS_REG = &SP_STATUS_REG;
		RspInfo10.SP__DMA_FULL_REG = &SP_DMA_FULL_REG;
		RspInfo11.SP__DMA_FULL_REG = &SP_DMA_FULL_REG;
		RspInfo10.SP__DMA_BUSY_REG = &SP_DMA_BUSY_REG;
		RspInfo11.SP__DMA_BUSY_REG = &SP_DMA_BUSY_REG;
		RspInfo10.SP__PC_REG = &SP_PC_REG;
		RspInfo11.SP__PC_REG = &SP_PC_REG;
		RspInfo10.SP__SEMAPHORE_REG = &SP_SEMAPHORE_REG;
		RspInfo11.SP__SEMAPHORE_REG = &SP_SEMAPHORE_REG;
			
		RspInfo10.DPC__START_REG = &DPC_START_REG;
		RspInfo11.DPC__START_REG = &DPC_START_REG;
		RspInfo10.DPC__END_REG = &DPC_END_REG;
		RspInfo11.DPC__END_REG = &DPC_END_REG;
		RspInfo10.DPC__CURRENT_REG = &DPC_CURRENT_REG;
		RspInfo11.DPC__CURRENT_REG = &DPC_CURRENT_REG;
		RspInfo10.DPC__STATUS_REG = &DPC_STATUS_REG;
		RspInfo11.DPC__STATUS_REG = &DPC_STATUS_REG;
		RspInfo10.DPC__CLOCK_REG = &DPC_CLOCK_REG;
		RspInfo11.DPC__CLOCK_REG = &DPC_CLOCK_REG;
		RspInfo10.DPC__BUFBUSY_REG = &DPC_BUFBUSY_REG;
		RspInfo11.DPC__BUFBUSY_REG = &DPC_BUFBUSY_REG;
		RspInfo10.DPC__PIPEBUSY_REG = &DPC_PIPEBUSY_REG;
		RspInfo11.DPC__PIPEBUSY_REG = &DPC_PIPEBUSY_REG;
		RspInfo10.DPC__TMEM_REG = &DPC_TMEM_REG;
		RspInfo11.DPC__TMEM_REG = &DPC_TMEM_REG;
		RspInfo10.OPT_SECTIONS = &RSPSECTIONS;
		RspInfo11.OPT_SECTIONS = &RSPSECTIONS;
				
		RspInfo11.IsSeeking = IsSeeking;

		if (RSPVersion == 0x0100) { InitiateRSP_1_0(RspInfo10, &RspTaskValue); }
		if (RSPVersion == 0x0101) { InitiateRSP_1_1(RspInfo11, &RspTaskValue); }
	}

	Controllers[0].Present = FALSE;
	Controllers[0].RawData = FALSE;
	Controllers[0].Plugin  = PLUGIN_NONE;
	
	Controllers[1].Present = FALSE;
	Controllers[1].RawData = FALSE;
	Controllers[1].Plugin  = PLUGIN_NONE;
	
	Controllers[2].Present = FALSE;
	Controllers[2].RawData = FALSE;
	Controllers[2].Plugin  = PLUGIN_NONE;
	
	Controllers[3].Present = FALSE;
	Controllers[3].RawData = FALSE;
	Controllers[3].Plugin  = PLUGIN_NONE;


	//EnableMenuItem(hMenu,ID_FILE_OPEN_ROM,MFS_ENABLED|MF_BYCOMMAND);		
	//if (GFXDllConfig) { EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_GFX,MFS_ENABLED|MF_BYCOMMAND); }
	//if (RSPDllConfig) { EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_RSP,MFS_ENABLED|MF_BYCOMMAND); }
	//if (AiDllConfig) { EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_AUDIO,MFS_ENABLED|MF_BYCOMMAND); }
	//if (ContConfig) { EnableMenuItem(hMenu,ID_OPTIONS_CONFIG_CONTROL,MFS_ENABLED|MF_BYCOMMAND); }

}

void ShutdownPlugins (void) {
	/*
	if (IsMenu(RspDebug.hRSPMenu)) {
		HMENU hMenu = GetMenu(hMainWindow);
		DeleteMenu(hMenu,(DWORD)RspDebug.hRSPMenu, MF_BYCOMMAND);		
	}
	if (IsMenu(GFXDebug.hGFXMenu)) {
		HMENU hMenu = GetMenu(hMainWindow);
		DeleteMenu(hMenu,(DWORD)GFXDebug.hGFXMenu, MF_BYCOMMAND);		
	}
	*/
	TerminateThread(hAudioThread,0);
	//if (GFXCloseDLL != NULL) { GFXCloseDLL(); }
	if (RSPCloseDLL != NULL) { RSPCloseDLL(); }
	if (AiCloseDLL != NULL) { AiCloseDLL(); }

	UnLoadRSPDll();

	//if (ContCloseDLL != NULL) { ContCloseDLL(); }
	//FreeLibrary(hAudioDll);
	//FreeLibrary(hControllerDll);
	//FreeLibrary(hGfxDll);
	//FreeLibrary(hRspDll);
}

BOOL ValidPluginVersion ( PLUGIN_INFO * PluginInfo ) {
	switch (PluginInfo->Type) {
	case PLUGIN_TYPE_RSP: 
		if (PluginInfo->Version == 0x0001) { return TRUE; }
		if (PluginInfo->Version == 0x0100) { return TRUE; }
		if (PluginInfo->Version == 0x0101) { return TRUE; }
		break;
	case PLUGIN_TYPE_GFX:
		if (PluginInfo->Version == 0x0102) { return TRUE; }
		if (PluginInfo->Version == 0x0103) { return TRUE; }
		break;
	case PLUGIN_TYPE_AUDIO:
		if (PluginInfo->Version == 0x0101) { return TRUE; }
		break;
	case PLUGIN_TYPE_CONTROLLER:
		if (PluginInfo->Version == 0x0100) { return TRUE; }
		break;
	}
	return FALSE;
}
